@@ -4,18 +4,16 @@ Provides public API.
 
 import os
 import io
-import sys
 import inspect
 import warnings
+import datetime
 import contextlib
 
-from distutils.version import LooseVersion
-
 from . import managers, exceptions, engines, utilities, resources
 from .version import __version__
 
 
-class Redmine(object):
+class Redmine:
     """
     Entry point for all requests.
     """
@@ -32,10 +30,24 @@ class Redmine(object):
         :param string datetime_format: (optional). Formatting directives for datetime format.
         :param raise_attr_exception: (optional). Control over resource attribute access exception raising.
         :type raise_attr_exception: bool or tuple
+        :param timezone: (optional). Whether to convert a naive datetime to a specific timezone aware one.
+        :type timezone: str or cls
         :param cls engine: (optional). Engine that will be used to make requests to Redmine.
         """
         self.url = url.rstrip('/')
         self.ver = kwargs.pop('version', None)
+
+        if self.ver is not None:
+            self.ver = utilities.versiontuple(self.ver)
+
+        self.timezone = kwargs.pop('timezone', None)
+
+        if self.timezone is not None and not isinstance(self.timezone, datetime.tzinfo):
+            try:
+                self.timezone = datetime.datetime.strptime(self.timezone, '%z').tzinfo
+            except (TypeError, ValueError):
+                raise exceptions.TimezoneError
+
         self.date_format = kwargs.pop('date_format', '%Y-%m-%d')
         self.datetime_format = kwargs.pop('datetime_format', '%Y-%m-%dT%H:%M:%SZ')
         self.raise_attr_exception = kwargs.pop('raise_attr_exception', True)
@@ -63,7 +75,7 @@ class Redmine(object):
         except KeyError:
             raise exceptions.ResourceError
 
-        if self.ver is not None and LooseVersion(str(self.ver)) < LooseVersion(resource_class.redmine_version):
+        if self.ver is not None and self.ver < resource_class.redmine_version:
             raise exceptions.ResourceVersionMismatchError
 
         return resource_class.manager_class(self, resource_class)
@@ -94,16 +106,16 @@ class Redmine(object):
         :type f: string or file-like object
         :param filename: (optional). Filename for the file that will be uploaded.
         """
-        if self.ver is not None and LooseVersion(str(self.ver)) < LooseVersion('1.4.0'):
+        if self.ver is not None and self.ver < (1, 4, 0):
             raise exceptions.VersionMismatchError('File uploading')
 
-        url = '{0}/uploads.json'.format(self.url)
+        url = f'{self.url}/uploads.json'
         headers = {'Content-Type': 'application/octet-stream'}
         params = {'filename': filename or ''}
 
-        # There're myriads of file-like object implementations here and there and some of them don't have
+        # There are myriads of file-like object implementations here and there and some of them don't have
         # a "read" method, which is wrong, but that's what we have, on the other hand it looks like all of
-        # them implement a "close" method, that's why we check for it here. Also we don't want to close the
+        # them implement a "close" method, that's why we check for it here. Also, we don't want to close the
         # stream ourselves as we have no idea of what the client is going to do with it afterwards, so we
         # leave the closing part to the client or to the garbage collector
         if hasattr(f, 'close'):
@@ -114,9 +126,9 @@ class Redmine(object):
 
             # We need to send bytes over the socket, so in case a file-like object contains a unicode
             # object underneath, we need to convert it to bytes, otherwise we'll get an exception
-            if isinstance(c, str if sys.version_info[0] >= 3 else unicode):
+            if isinstance(c, str):
                 warnings.warn("File-like object contains unicode, hence an additional step is performed to convert "
-                              "it's content to bytes, please consider switching to bytes to eliminate this warning",
+                              "its content to bytes, please consider switching to bytes to eliminate this warning",
                               exceptions.PerformanceWarning)
                 f = io.BytesIO(f.read().encode('utf-8'))
 
@@ -154,10 +166,7 @@ class Redmine(object):
         if savepath is None:
             return response
 
-        try:
-            from urlparse import urlsplit
-        except ImportError:
-            from urllib.parse import urlsplit
+        from urllib.parse import urlsplit
 
         if filename is None:
             filename = urlsplit(url)[2].split('/')[-1]
@@ -186,7 +195,7 @@ class Redmine(object):
         :param string query: (required). What to search.
         :param dict options: (optional). Dictionary of search options.
         """
-        if self.ver is not None and LooseVersion(str(self.ver)) < LooseVersion('3.0.0'):
+        if self.ver is not None and self.ver < (3, 0, 0):
             raise exceptions.VersionMismatchError('Search functionality')
 
         container_map, manager_map, results = {}, {}, {'unknown': {}}
@@ -205,7 +214,7 @@ class Redmine(object):
 
                 manager_map[container] = getattr(self, name)
 
-        raw_resources, _ = self.engine.bulk_request('get', '{0}/search.json'.format(self.url), 'results', **options)
+        raw_resources, _ = self.engine.bulk_request('get', f'{self.url}/search.json', 'results', **options)
 
         for resource in raw_resources:
             if resource['type'] in container_map:
@@ -221,7 +230,7 @@ class Redmine(object):
 
                 results['unknown'][resource['type']].append(resource)
 
-            del resource['type']  # all resources are already sorted by type so we don't need it
+            del resource['type']  # all resources are already sorted by type, so we don't need it
 
         if not results['unknown']:
             del results['unknown']
