@@ -1,25 +1,24 @@
 """
-Defines base Redmine resource class and it's infrastructure.
+Defines base Redmine resource class and its infrastructure.
 """
 
-from __future__ import unicode_literals
-
-from datetime import date, datetime
+from datetime import date, datetime, timezone
 
 from .. import managers, utilities, exceptions
 
-
 registry = {}
 
 
 class Registrar(type):
     """
     A resource that implements this metaclass, i.e. all resources that inherit from BaseResource,
-    will be added to a resource registry to be managed by it's ResourceManager. Resource classes
+    will be added to a resource registry to be managed by its ResourceManager. Resource classes
     which name starts with Base are considered base classes and not added to the registry.
     """
     def __new__(mcs, name, bases, attrs):
-        cls = super(Registrar, mcs).__new__(mcs, name, bases, attrs)
+        mcs.update_query_strings(attrs)
+
+        cls = super().__new__(mcs, name, bases, attrs)
 
         if name.startswith('Base'):  # base classes shouldn't be added to the registry
             return cls
@@ -27,13 +26,16 @@ class Registrar(type):
         if name not in registry:  # a name maybe already added to registry by other classes
             registry[name] = {}
 
-        for attr in ('_attach_includes', '_attach_relations'):
+        for attr in ('_attach_includes', '_attach_relations', '_attach_includes_map'):
             class_attr_name = attr[7:]
             registry_attr_name = attr[1:]
 
             if registry_attr_name in registry[name]:
-                mcs.update_cls_attr(cls, class_attr_name, registry[name][registry_attr_name].keys())
-                mcs.update_cls_attr(cls, '_resource_set_map', registry[name][registry_attr_name])
+                if attr == '_attach_includes_map':
+                    mcs.update_cls_attr(cls, class_attr_name, dict(registry[name][registry_attr_name].keys()))
+                else:
+                    mcs.update_cls_attr(cls, class_attr_name, registry[name][registry_attr_name].keys())
+                    mcs.update_cls_attr(cls, '_resource_set_map', registry[name][registry_attr_name])
 
             if not isinstance(getattr(cls, attr), dict):
                 continue
@@ -48,11 +50,25 @@ class Registrar(type):
                 registry[resource_name][registry_attr_name][value] = name
 
                 if 'class' in registry[resource_name]:
-                    mcs.update_cls_attr(registry[resource_name]['class'], class_attr_name, [value])
-                    mcs.update_cls_attr(registry[resource_name]['class'], '_resource_set_map', {value: name})
+                    if attr == '_attach_includes_map':
+                        mcs.update_cls_attr(registry[resource_name]['class'], class_attr_name, dict([value]))
+                    else:
+                        mcs.update_cls_attr(registry[resource_name]['class'], class_attr_name, [value])
+                        mcs.update_cls_attr(registry[resource_name]['class'], '_resource_set_map', {value: name})
 
         return registry[name].setdefault('class', cls)
 
+    @staticmethod
+    def update_query_strings(attrs):
+        """
+        Updates all `query_*` string attributes to use ResourceQueryFormatter by default.
+        """
+        for k, v in attrs.items():
+            if k.startswith('query_') and v is not None:
+                attrs[k] = utilities.ResourceQueryStr(v)
+
+        return attrs
+
     @staticmethod
     def update_cls_attr(cls, name, value):
         """
@@ -75,8 +91,7 @@ class Registrar(type):
         setattr(cls, name, value)
 
 
-@utilities.fix_unicode
-class BaseResource(utilities.with_metaclass(Registrar)):
+class BaseResource(metaclass=Registrar):
     """
     Implementation of Redmine resource.
     """
@@ -96,7 +111,9 @@ class BaseResource(utilities.with_metaclass(Registrar)):
     query_create = None
     query_update = None
     query_delete = None
+    query_url = None
     search_hints = None
+    extra_export_columns = []
     http_method_create = 'post'
     http_method_update = 'put'
     http_method_delete = 'delete'
@@ -104,6 +121,7 @@ class BaseResource(utilities.with_metaclass(Registrar)):
 
     _repr = [['id', 'name']]
     _includes = []
+    _includes_map = {}
     _relations = []
     _relations_name = None
     _unconvertible = ['name', 'description']
@@ -111,6 +129,7 @@ class BaseResource(utilities.with_metaclass(Registrar)):
     _create_readonly = ['id', 'created_on', 'updated_on', 'author', 'user', 'project', 'issue']
     _update_readonly = _create_readonly[:]
     _attach_includes = None
+    _attach_includes_map = None
     _attach_relations = None
     _resource_map = {}  # Resources that should become a Resource object
     _resource_set_map = {}  # Resources that should become a ResourceSet object
@@ -125,8 +144,8 @@ class BaseResource(utilities.with_metaclass(Registrar)):
         relations_includes = self._relations + self._includes
 
         self.manager = manager
-        self._create_readonly += relations_includes
-        self._update_readonly += relations_includes
+        self._create_readonly = self._create_readonly[:] + relations_includes
+        self._update_readonly = self._update_readonly[:] + relations_includes
         self._decoded_attrs = dict(dict.fromkeys(relations_includes), **attributes)
         self._encoded_attrs = {}
         self._changes = {}
@@ -136,7 +155,7 @@ class BaseResource(utilities.with_metaclass(Registrar)):
 
     def __getitem__(self, item):
         """
-        Provides a dictionary-like access to Resource attributes.
+        Provides dictionary-like access to Resource attributes.
         """
         return getattr(self, item)
 
@@ -154,24 +173,26 @@ class BaseResource(utilities.with_metaclass(Registrar)):
             raise AttributeError
 
         # If this isn't the first time attribute access we can return it from cache
-        encoded = self._encoded_attrs.get(attr)
-        if encoded is not None:
-            return encoded
+        if attr in self._encoded_attrs:
+            return self._encoded_attrs[attr]
 
-        # Else this is the first time access and we need to encode the attribute
-        decoded = self._decoded_attrs.get(attr)
-        if decoded is not None:
-            attr, encoded = self.encode(attr, decoded, self.manager)
-        elif attr in self._relations:
-            filters = {'{0}_id'.format(self._relations_name): self.internal_id}
-            encoded = self.manager.new_manager(self._resource_set_map[attr]).filter(**filters)
+        # Else this is the first time access hence we need to encode the attribute
+        if attr in self._relations:
+            filters = {f'{self._relations_name}_id': self.internal_id}
+            self._encoded_attrs[attr] = self.manager.new_manager(self._resource_set_map[attr]).filter(**filters)
         elif attr in self._includes:
-            attr, encoded = self.encode(attr, self.refresh(itself=False, include=attr).raw()[attr] or [], self.manager)
+            value = self._decoded_attrs[attr] = self._decoded_attrs.pop(self._includes_map.get(attr, attr), None)
+
+            if value is None:
+                value = self.refresh(itself=False, include=attr).raw().get(self._includes_map.get(attr, attr)) or []
+
+            self._encoded_attrs.update([self.encode(attr, value, self.manager)])
+        elif attr in self._decoded_attrs:
+            self._encoded_attrs.update([self.encode(attr, self._decoded_attrs[attr], self.manager)])
 
         # In case of successful encoding we put it to a cache and return
-        if encoded is not None:
-            self._encoded_attrs[attr] = encoded
-            return encoded
+        if attr in self._encoded_attrs:
+            return self._encoded_attrs[attr]
 
         # Else we return the defaults if this is a new item or throw an exception
         if self.is_new():
@@ -191,7 +212,7 @@ class BaseResource(utilities.with_metaclass(Registrar)):
         Sets the requested attribute.
         """
         if attr in self._members or attr.startswith('_'):
-            return super(BaseResource, self).__setattr__(attr, value)
+            return super().__setattr__(attr, value)
         elif attr in self._create_readonly and self.is_new():
             raise exceptions.ReadonlyAttrError
         elif attr in self._update_readonly and not self.is_new():
@@ -218,7 +239,7 @@ class BaseResource(utilities.with_metaclass(Registrar)):
             elif attr in self._multiple_attr_id_map:
                 self._decoded_attrs[self._multiple_attr_id_map[attr]] = [{'id': attr_id} for attr_id in decoded_value]
 
-        # When we set an attribute we put it's decoded value only to a _decoded_attrs
+        # When we set an attribute we put its decoded value only to a _decoded_attrs
         # dict because it may never be accessed again, that is why we don't waste time
         # on the encode process but only clean the cache, and in case if it will be
         # accessed, the encoding process will be run automatically by __getattr__
@@ -238,12 +259,16 @@ class BaseResource(utilities.with_metaclass(Registrar)):
         if type_ is date:
             return attr, value.strftime(manager.redmine.date_format)
         elif type_ is datetime:
+            if manager.redmine.timezone is not None and value.tzinfo is not None:
+                value = value.astimezone(timezone.utc)
+
             return attr, value.strftime(manager.redmine.datetime_format)
 
         if attr == 'uploads':
             for index, attachment in enumerate(value):
                 if 'token' not in attachment:
-                    value[index]['token'] = manager.redmine.upload(attachment.pop('path', ''))['token']
+                    value[index]['token'] = manager.redmine.upload(
+                        attachment.pop('path', ''), filename=attachment.get('filename'))['token']
 
             return attr, value
         elif attr == 'include' and isinstance(value, (list, tuple)):
@@ -271,7 +296,12 @@ class BaseResource(utilities.with_metaclass(Registrar)):
 
         try:
             try:
-                return attr, datetime.strptime(value, manager.redmine.datetime_format)
+                value = datetime.strptime(value, manager.redmine.datetime_format)
+
+                if manager.redmine.timezone is not None:
+                    value = value.replace(tzinfo=timezone.utc).astimezone(manager.redmine.timezone)
+
+                return attr, value
             except (TypeError, ValueError):
                 return attr, datetime.strptime(value, manager.redmine.date_format).date()
         except (TypeError, ValueError):
@@ -370,7 +400,11 @@ class BaseResource(utilities.with_metaclass(Registrar)):
             self.post_update()
         else:
             self.pre_create()
-            self._decoded_attrs = self.manager.create(**self._changes).raw()
+            resource = self.manager.create(**self._changes)
+
+            if resource is not None:
+                self._decoded_attrs = resource.raw()
+
             self.post_create()
 
         self._changes = {}
@@ -423,7 +457,9 @@ class BaseResource(utilities.with_metaclass(Registrar)):
         """
         Returns full URL to the Resource for humans if there is one.
         """
-        if self.query_one is not None:
+        if self.query_url is not None:
+            return self.manager.redmine.url + self.query_url.format(self.internal_id)
+        elif self.query_one is not None:
             return self.manager.redmine.url + self.query_one.format(self.internal_id)[:-5]
 
         return None
@@ -437,7 +473,7 @@ class BaseResource(utilities.with_metaclass(Registrar)):
 
     def is_new(self):
         """
-        Checks if Resource was just created and not yet saved to Redmine or it is an existing Resource.
+        Checks if Resource was just created and not yet saved to Redmine, or it is an existing Resource.
         """
         return False if 'id' in self._decoded_attrs or 'created_on' in self._decoded_attrs else True
 
@@ -476,7 +512,7 @@ class BaseResource(utilities.with_metaclass(Registrar)):
                 _repr_.insert(0, value)
 
                 if attr != 'id':
-                    _str_.insert(0, value)
+                    _str_.insert(0, str(value))
 
             if len(_repr_) > 0:
                 break
@@ -498,12 +534,12 @@ class BaseResource(utilities.with_metaclass(Registrar)):
         Official representation of a Resource object.
         """
         values = self._representation('repr')
-        view = '<redminelib.resources.{0.__class__.__name__}'.format(self)
+        view = f'<redminelib.resources.{self.__class__.__name__}'
 
         if isinstance(values[0], int):
-            view += ' #{0}'.format(values.pop(0))
+            view += f' #{values.pop(0)}'
 
         if len(values) > 0:
-            view += ' "{0}"'.format(' '.join(values))
+            view += f" \"{' '.join(values)}\""
 
         return view + '>'
