@@ -2,34 +2,32 @@
 Defines standard Redmine resources and resource mappings.
 """
 
-from __future__ import unicode_literals
-
-from distutils.version import LooseVersion
-
 from . import BaseResource
 from .. import managers, exceptions
 
 
 class Project(BaseResource):
-    redmine_version = '1.0'
+    redmine_version = (1, 0, 0)
     container_all = 'projects'
     container_one = 'project'
     container_create = 'project'
     container_update = 'project'
     query_all_export = '/projects.{format}'
     query_all = '/projects.json'
-    query_one = '/projects/{0}.json'
+    query_one = '/projects/{}.json'
     query_create = '/projects.json'
-    query_update = '/projects/{0}.json'
-    query_delete = '/projects/{0}.json'
+    query_update = '/projects/{}.json'
+    query_delete = '/projects/{}.json'
     search_hints = ['project']
+    manager_class = managers.ProjectManager
 
     _repr = [['id', 'name'], ['title']]
-    _includes = ['trackers', 'issue_categories', 'enabled_modules', 'time_entry_activities']
+    _includes = ['trackers', 'issue_categories', 'enabled_modules', 'time_entry_activities', 'issue_custom_fields']
     _relations = ['wiki_pages', 'memberships', 'issue_categories', 'time_entries', 'versions',
                   'news', 'issues', 'files']
     _unconvertible = BaseResource._unconvertible + ['identifier', 'status']
     _update_readonly = BaseResource._update_readonly + ['identifier']
+    _resource_map = {'default_version': 'Version', 'default_assignee': 'User'}
     _resource_set_map = {
         'custom_fields': 'CustomField',
         'trackers': 'Tracker',
@@ -41,6 +39,8 @@ class Project(BaseResource):
         'news': 'News',
         'issues': 'Issue',
         'files': 'File',
+        'time_entry_activities': 'Enumeration',
+        'issue_custom_fields': 'CustomField',
     }
     _single_attr_id_map = {'parent_id': 'parent'}
     _multiple_attr_id_map = {'tracker_ids': 'trackers'}
@@ -54,28 +54,35 @@ class Project(BaseResource):
         if attr == 'enabled_modules':
             return attr, [module['name'] for module in value]
 
-        return super(Project, cls).encode(attr, value, manager)
+        return super().encode(attr, value, manager)
+
+    def __getattr__(self, attr):
+        if attr in ('close', 'reopen', 'archive', 'unarchive'):
+            return lambda: getattr(self.manager, attr)(self.internal_id)
+
+        return super().__getattr__(attr)
 
 
 class Issue(BaseResource):
-    redmine_version = '1.0'
+    redmine_version = (1, 0, 0)
     container_all = 'issues'
     container_one = 'issue'
     container_filter = 'issues'
     container_create = 'issue'
     container_update = 'issue'
     query_all_export = '/issues.{format}'
-    query_one_export = '/issues/{0}.{format}'
+    query_one_export = '/issues/{}.{format}'
     query_all = '/issues.json?status_id=*'
-    query_one = '/issues/{0}.json'
+    query_one = '/issues/{}.json'
     query_filter = '/issues.json'
     query_create = '/projects/{project_id}/issues.json'
-    query_update = '/issues/{0}.json'
-    query_delete = '/issues/{0}.json'
+    query_update = '/issues/{}.json'
+    query_delete = '/issues/{}.json'
     search_hints = ['issue', 'issue closed']
+    extra_export_columns = ['description', 'last_notes']
 
     _repr = [['id', 'subject'], ['title'], ['id']]
-    _includes = ['children', 'attachments', 'relations', 'changesets', 'journals', 'watchers']
+    _includes = ['children', 'attachments', 'relations', 'changesets', 'journals', 'watchers', 'allowed_statuses']
     _relations = ['relations', 'time_entries']
     _unconvertible = BaseResource._unconvertible + ['subject', 'notes']
     _create_readonly = BaseResource._create_readonly + ['spent_hours']
@@ -98,6 +105,7 @@ class Issue(BaseResource):
         'relations': 'IssueRelation',
         'watchers': 'User',
         'time_entries': 'TimeEntry',
+        'allowed_statuses': 'IssueStatus',
     }
     _single_attr_id_map = {
         'project_id': 'project',
@@ -119,7 +127,7 @@ class Issue(BaseResource):
             self._redmine = issue.manager.redmine
             self._issue_id = issue.internal_id
 
-            if self._redmine.ver is not None and LooseVersion(str(self._redmine.ver)) < LooseVersion('2.3'):
+            if self._redmine.ver is not None and self._redmine.ver < (2, 3, 0):
                 raise exceptions.ResourceVersionMismatchError
 
         def add(self, user_id):
@@ -128,7 +136,7 @@ class Issue(BaseResource):
 
             :param int user_id: (required). User id.
             """
-            url = '{0}/issues/{1}/watchers.json'.format(self._redmine.url, self._issue_id)
+            url = f'{self._redmine.url}/issues/{self._issue_id}/watchers.json'
             return self._redmine.engine.request('post', url, data={'user_id': user_id})
 
         def remove(self, user_id):
@@ -137,7 +145,7 @@ class Issue(BaseResource):
 
             :param int user_id: (required). User id.
             """
-            url = '{0}/issues/{1}/watchers/{2}.json'.format(self._redmine.url, self._issue_id, user_id)
+            url = f'{self._redmine.url}/issues/{self._issue_id}/watchers/{user_id}.json'
             return self._redmine.engine.request('delete', url)
 
     def __getattr__(self, attr):
@@ -147,13 +155,13 @@ class Issue(BaseResource):
         if attr == 'version':
             attr = 'fixed_version'
 
-        return super(Issue, self).__getattr__(attr)
+        return super().__getattr__(attr)
 
     def __setattr__(self, attr, value):
         if attr == 'version_id':
             attr = 'fixed_version_id'
 
-        super(Issue, self).__setattr__(attr, value)
+        super().__setattr__(attr, value)
 
     @classmethod
     def decode(cls, attr, value, manager):
@@ -164,11 +172,11 @@ class Issue(BaseResource):
         elif attr == 'checklists':
             return 'checklists_attributes', value
 
-        return super(Issue, cls).decode(attr, value, manager)
+        return super().decode(attr, value, manager)
 
 
 class TimeEntry(BaseResource):
-    redmine_version = '1.1'
+    redmine_version = (1, 1, 0)
     container_all = 'time_entries'
     container_one = 'time_entry'
     container_filter = 'time_entries'
@@ -176,16 +184,16 @@ class TimeEntry(BaseResource):
     container_update = 'time_entry'
     query_all_export = '/time_entries.{format}'
     query_all = '/time_entries.json'
-    query_one = '/time_entries/{0}.json'
+    query_one = '/time_entries/{}.json'
     query_filter = '/time_entries.json'
     query_create = '/time_entries.json'
-    query_update = '/time_entries/{0}.json'
-    query_delete = '/time_entries/{0}.json'
+    query_update = '/time_entries/{}.json'
+    query_delete = '/time_entries/{}.json'
 
     _repr = [['id']]
     _resource_map = {'project': 'Project', 'issue': 'Issue', 'user': 'User', 'activity': 'Enumeration'}
     _resource_set_map = {'custom_fields': 'CustomField'}
-    _single_attr_id_map = {'issue_id': 'issue', 'activity_id': 'activity'}
+    _single_attr_id_map = {'project_id': 'project', 'issue_id': 'issue', 'activity_id': 'activity'}
 
     @classmethod
     def decode(cls, attr, value, manager):
@@ -194,28 +202,25 @@ class TimeEntry(BaseResource):
         elif attr == 'to_date':
             attr = 'to'
 
-        return super(TimeEntry, cls).decode(attr, value, manager)
+        return super().decode(attr, value, manager)
 
 
 class Enumeration(BaseResource):
-    redmine_version = '2.2'
+    redmine_version = (2, 2, 0)
     container_filter = '{resource}'
     query_filter = '/enumerations/{resource}.json'
+    query_url = '/enumerations/{}/edit'
 
     _resource_set_map = {'custom_fields': 'CustomField'}
 
-    @property
-    def url(self):
-        return '{0}/enumerations/{1}/edit'.format(self.manager.redmine.url, self.internal_id)
-
 
 class Attachment(BaseResource):
-    redmine_version = '1.3'
+    redmine_version = (1, 3, 0)
     container_one = 'attachment'
     container_update = 'attachment'
-    query_one = '/attachments/{0}.json'
-    query_update = '/attachments/{0}.json'
-    query_delete = '/attachments/{0}.json'
+    query_one = '/attachments/{}.json'
+    query_update = '/attachments/{}.json'
+    query_delete = '/attachments/{}.json'
     http_method_update = 'patch'
 
     _repr = [['id', 'filename'], ['id']]
@@ -226,7 +231,7 @@ class Attachment(BaseResource):
 
 
 class File(Attachment):
-    redmine_version = '3.4'
+    redmine_version = (3, 4, 0)
     container_filter = 'files'
     container_create = 'file'
     query_filter = '/projects/{project_id}/files.json'
@@ -240,11 +245,13 @@ class File(Attachment):
         if attr == 'path':
             return 'token', manager.redmine.upload(value)['token']
 
-        return super(File, cls).decode(attr, value, manager)
+        return super().decode(attr, value, manager)
 
 
 class IssueJournal(BaseResource):
-    redmine_version = '1.0'
+    redmine_version = (1, 0, 0)
+    container_update = 'journal'
+    query_update = '/journals/{}.json'
 
     _repr = [['id']]
     _unconvertible = ['notes']
@@ -253,17 +260,17 @@ class IssueJournal(BaseResource):
 
 class WikiPage(BaseResource):
     internal_id_key = 'title'
-    redmine_version = '2.2'
+    redmine_version = (2, 2, 0)
     container_filter = 'wiki_pages'
     container_one = 'wiki_page'
     container_create = 'wiki_page'
     container_update = 'wiki_page'
-    query_one_export = '/projects/{project_id}/wiki/{0}.{format}'
+    query_one_export = '/projects/{project_id}/wiki/{}.{format}'
     query_filter = '/projects/{project_id}/wiki/index.json'
-    query_one = '/projects/{project_id}/wiki/{0}.json'
+    query_one = '/projects/{project_id}/wiki/{}.json'
     query_create = '/projects/{project_id}/wiki/{title}.json'
-    query_update = '/projects/{project_id}/wiki/{0}.json'
-    query_delete = '/projects/{project_id}/wiki/{0}.json'
+    query_update = '/projects/{project_id}/wiki/{}.json'
+    query_delete = '/projects/{project_id}/wiki/{}.json'
     search_hints = ['wiki-page']
     http_method_create = 'put'
     manager_class = managers.WikiPageManager
@@ -283,16 +290,16 @@ class WikiPage(BaseResource):
             value = manager.new_manager(cls.__name__, project_id=manager.params.get('project_id', 0)).to_resource(value)
             return attr, value
 
-        return super(WikiPage, cls).encode(attr, value, manager)
+        return super().encode(attr, value, manager)
 
     def refresh(self, **params):
-        return super(WikiPage, self).refresh(**dict(params, project_id=self.project_id))
+        return super().refresh(**dict(params, project_id=self.project_id))
 
     def post_update(self):
         self._encoded_attrs['version'] = self._decoded_attrs['version'] = self._decoded_attrs.get('version', 0) + 1
 
     def delete(self, **params):
-        return super(WikiPage, self).delete(**dict(params, project_id=self.project_id))
+        return super().delete(**dict(params, project_id=self.project_id))
 
     def export_url(self, fmt):
         return self.manager.redmine.url + self.query_one_export.format(
@@ -312,73 +319,74 @@ class WikiPage(BaseResource):
         if attr == 'text' and attr not in self._decoded_attrs:
             self._decoded_attrs[attr] = self.refresh(itself=False).raw()[attr]
 
-        return super(WikiPage, self).__getattr__(attr)
+        return super().__getattr__(attr)
 
     def __int__(self):
         return self.version
 
 
 class ProjectMembership(BaseResource):
-    redmine_version = '1.4'
+    redmine_version = (1, 4, 0)
     container_filter = 'memberships'
     container_one = 'membership'
     container_update = 'membership'
     container_create = 'membership'
     query_filter = '/projects/{project_id}/memberships.json'
-    query_one = '/memberships/{0}.json'
+    query_one = '/memberships/{}.json'
     query_create = '/projects/{project_id}/memberships.json'
-    query_update = '/memberships/{0}.json'
-    query_delete = '/memberships/{0}.json'
+    query_update = '/memberships/{}.json'
+    query_delete = '/memberships/{}.json'
 
     _repr = [['id']]
     _create_readonly = BaseResource._create_readonly + ['user', 'roles']
     _update_readonly = _create_readonly[:]
     _resource_map = {'project': 'Project', 'user': 'User', 'group': 'Group'}
     _resource_set_map = {'roles': 'Role'}
-    _single_attr_id_map = {'project_id': 'project', 'user_id': 'users'}
+    _single_attr_id_map = {'project_id': 'project', 'user_id': 'user'}
     _multiple_attr_id_map = {'role_ids': 'roles'}
 
 
 class IssueCategory(BaseResource):
-    redmine_version = '1.3'
+    redmine_version = (1, 3, 0)
     container_filter = 'issue_categories'
     container_one = 'issue_category'
     container_update = 'issue_category'
     container_create = 'issue_category'
     query_filter = '/projects/{project_id}/issue_categories.json'
-    query_one = '/issue_categories/{0}.json'
+    query_one = '/issue_categories/{}.json'
     query_create = '/projects/{project_id}/issue_categories.json'
-    query_update = '/issue_categories/{0}.json'
-    query_delete = '/issue_categories/{0}.json'
+    query_update = '/issue_categories/{}.json'
+    query_delete = '/issue_categories/{}.json'
 
     _resource_map = {'project': 'Project', 'assigned_to': 'User'}
+    _single_attr_id_map = {'project_id': 'project', 'assigned_to_id': 'assigned_to'}
 
 
 class IssueRelation(BaseResource):
-    redmine_version = '1.3'
+    redmine_version = (1, 3, 0)
     container_filter = 'relations'
     container_one = 'relation'
     container_create = 'relation'
     query_filter = '/issues/{issue_id}/relations.json'
-    query_one = '/relations/{0}.json'
+    query_one = '/relations/{}.json'
     query_create = '/issues/{issue_id}/relations.json'
-    query_delete = '/relations/{0}.json'
+    query_delete = '/relations/{}.json'
 
     _repr = [['id']]
     _single_attr_id_map = {'issue_id': 'issue'}
 
 
 class Version(BaseResource):
-    redmine_version = '1.3'
+    redmine_version = (1, 3, 0)
     container_filter = 'versions'
     container_one = 'version'
     container_create = 'version'
     container_update = 'version'
     query_filter = '/projects/{project_id}/versions.json'
-    query_one = '/versions/{0}.json'
+    query_one = '/versions/{}.json'
     query_create = '/projects/{project_id}/versions.json'
-    query_update = '/versions/{0}.json'
-    query_delete = '/versions/{0}.json'
+    query_update = '/versions/{}.json'
+    query_delete = '/versions/{}.json'
 
     _unconvertible = ['status']
     _resource_map = {'project': 'Project'}
@@ -387,18 +395,20 @@ class Version(BaseResource):
 
 
 class User(BaseResource):
-    redmine_version = '1.1'
+    redmine_version = (1, 1, 0)
     container_all = 'users'
     container_one = 'user'
     container_filter = 'users'
     container_create = 'user'
     container_update = 'user'
+    query_all_export = '/users.{format}'
     query_all = '/users.json'
-    query_one = '/users/{0}.json'
+    query_one = '/users/{}.json'
     query_filter = '/users.json'
     query_create = '/users.json'
-    query_update = '/users/{0}.json'
-    query_delete = '/users/{0}.json'
+    query_update = '/users/{}.json'
+    query_delete = '/users/{}.json'
+    manager_class = managers.UserManager
 
     _repr = [['id', 'firstname', 'lastname'], ['id', 'name']]
     _includes = ['memberships', 'groups']
@@ -418,24 +428,24 @@ class User(BaseResource):
     def __getattr__(self, attr):
         if attr == 'time_entries' and attr not in self._encoded_attrs:
             self._relations_name = 'user'
-            value = super(User, self).__getattr__(attr)
+            value = super().__getattr__(attr)
             self._relations_name = 'assigned_to'
             return value
 
-        return super(User, self).__getattr__(attr)
+        return super().__getattr__(attr)
 
 
 class Group(BaseResource):
-    redmine_version = '2.1'
+    redmine_version = (2, 1, 0)
     container_all = 'groups'
     container_one = 'group'
     container_create = 'group'
     container_update = 'group'
     query_all = '/groups.json'
-    query_one = '/groups/{0}.json'
+    query_one = '/groups/{}.json'
     query_create = '/groups.json'
-    query_update = '/groups/{0}.json'
-    query_delete = '/groups/{0}.json'
+    query_update = '/groups/{}.json'
+    query_delete = '/groups/{}.json'
 
     _includes = ['memberships', 'users']
     _resource_set_map = {'memberships': 'ProjectMembership', 'users': 'User', 'custom_fields': 'CustomField'}
@@ -455,7 +465,7 @@ class Group(BaseResource):
 
             :param int user_id: (required). User id.
             """
-            url = '{0}/groups/{1}/users.json'.format(self._redmine.url, self._group_id)
+            url = f'{self._redmine.url}/groups/{self._group_id}/users.json'
             return self._redmine.engine.request('post', url, data={'user_id': user_id})
 
         def remove(self, user_id):
@@ -464,83 +474,86 @@ class Group(BaseResource):
 
             :param int user_id: (required). User id.
             """
-            url = '{0}/groups/{1}/users/{2}.json'.format(self._redmine.url, self._group_id, user_id)
+            url = f'{self._redmine.url}/groups/{self._group_id}/users/{user_id}.json'
             return self._redmine.engine.request('delete', url)
 
     def __getattr__(self, attr):
         if attr == 'user':
             return Group.User(self)
 
-        return super(Group, self).__getattr__(attr)
+        return super().__getattr__(attr)
 
 
 class Role(BaseResource):
-    redmine_version = '1.4'
+    redmine_version = (1, 4, 0)
     container_all = 'roles'
     container_one = 'role'
     query_all = '/roles.json'
-    query_one = '/roles/{0}.json'
+    query_one = '/roles/{}.json'
 
 
 class News(BaseResource):
-    redmine_version = '1.1'
+    redmine_version = (1, 1, 0)
     container_all = 'news'
+    container_one = 'news'
     container_filter = 'news'
+    container_create = 'news'
+    container_update = 'news'
     query_all_export = '/news.{format}'
     query_all = '/news.json'
+    query_one = '/news/{}.json'
     query_filter = '/news.json'
+    query_create = '/projects/{project_id}/news.json'
+    query_update = '/news/{}.json'
+    query_delete = '/news/{}.json'
     search_hints = ['news']
+    manager_class = managers.NewsManager
 
     _repr = [['id', 'title']]
+    _includes = ['attachments', 'comments']
     _resource_map = {'project': 'Project', 'author': 'User'}
-
-    @property
-    def url(self):
-        return '{0}/news/{1}'.format(self.manager.redmine.url, self.internal_id)
+    _resource_set_map = {'attachments': 'Attachment'}
+    _single_attr_id_map = {'project_id': 'project'}
 
 
 class IssueStatus(BaseResource):
-    redmine_version = '1.3'
+    redmine_version = (1, 3, 0)
     container_all = 'issue_statuses'
     query_all = '/issue_statuses.json'
+    query_url = '/issue_statuses/{}/edit'
 
     _relations = ['issues']
     _relations_name = 'status'
     _resource_set_map = {'issues': 'Issue'}
 
-    @property
-    def url(self):
-        return '{0}/issue_statuses/{1}/edit'.format(self.manager.redmine.url, self.internal_id)
-
 
 class Tracker(BaseResource):
-    redmine_version = '1.3'
+    redmine_version = (1, 3, 0)
     container_all = 'trackers'
     query_all = '/trackers.json'
+    query_url = '/trackers/{}/edit'
 
     _relations = ['issues']
     _resource_set_map = {'issues': 'Issue'}
 
-    @property
-    def url(self):
-        return '{0}/trackers/{1}/edit'.format(self.manager.redmine.url, self.internal_id)
-
 
 class Query(BaseResource):
-    redmine_version = '1.3'
+    redmine_version = (1, 3, 0)
     container_all = 'queries'
     query_all = '/queries.json'
+    query_url = '/projects/{}/issues?query_id={}'
 
     @property
     def url(self):
-        return '{0}/projects/{1}/issues?query_id={2}'.format(
-            self.manager.redmine.url, self._decoded_attrs.get('project_id', 0), self.internal_id)
+        return self.manager.redmine.url + self.query_url.format(
+            self._decoded_attrs.get('project_id', 0), self.internal_id)
 
 
 class CustomField(BaseResource):
-    redmine_version = '2.4'
+    redmine_version = (2, 4, 0)
     container_all = 'custom_fields'
     query_all = '/custom_fields.json'
+    query_url = '/custom_fields/{}/edit'
 
     _resource_set_map = {'trackers': 'Tracker', 'roles': 'Role'}
 
@@ -552,7 +565,7 @@ class CustomField(BaseResource):
         if attr == 'value' and attr not in self._decoded_attrs:
             return ''
 
-        return super(CustomField, self).__getattr__(attr)
+        return super().__getattr__(attr)
 
     @classmethod
     def encode(cls, attr, value, manager):
@@ -562,8 +575,4 @@ class CustomField(BaseResource):
         if attr == 'trackers' and 'tracker' in value:
             value = [value['tracker']]
 
-        return super(CustomField, cls).encode(attr, value, manager)
-
-    @property
-    def url(self):
-        return '{0}/custom_fields/{1}/edit'.format(self.manager.redmine.url, self.internal_id)
+        return super().encode(attr, value, manager)
