@@ -21,7 +21,7 @@ class ResourceManagerTestCase(BaseRedmineTestCase):
         self.assertRaises(exceptions.ResourceError, lambda: self.redmine.foobar)
 
     def test_not_supported_version_exception(self):
-        self.redmine.ver = '0.0.1'
+        self.redmine.ver = (0, 0, 1)
         self.assertRaises(exceptions.ResourceVersionMismatchError, lambda: self.redmine.project)
 
     def test_convert_dict_to_resource_object(self):
@@ -254,7 +254,7 @@ class ResourceManagerTestCase(BaseRedmineTestCase):
         self.assertRaises(exceptions.ResourceNotFoundError, lambda: list(self.redmine.project.all()))
 
     def test_resource_requirements_exception(self):
-        FooResource.requirements = ('foo plugin', ('bar plugin', '1.2.3'),)
+        FooResource.requirements = ('foo plugin', ('bar plugin', (1, 2, 3)),)
         self.response.status_code = 404
         self.assertRaises(exceptions.ResourceRequirementsError, lambda: self.redmine.foo_resource.get(1))
         self.assertRaises(exceptions.ResourceRequirementsError, lambda: list(self.redmine.foo_resource.all()))
