@@ -13,11 +13,11 @@ class StandardResourcesTestCase(BaseRedmineTestCase):
 
     def test_supports_url_retrieval(self):
         self.response.json.return_value = responses['project']['get']
-        self.assertEqual(self.redmine.project.get(1).url, '{0}/projects/foo'.format(self.url))
+        self.assertEqual(self.redmine.project.get(1).url, f'{self.url}/projects/foo')
 
     def test_supports_export_url_retrieval(self):
         self.response.json.return_value = responses['issue']['get']
-        self.assertEqual(self.redmine.issue.get(1).export_url('pdf'), '{0}/issues/1.pdf'.format(self.url))
+        self.assertEqual(self.redmine.issue.get(1).export_url('pdf'), f'{self.url}/issues/1.pdf')
         self.response.json.return_value = responses['attachment']['get']
         self.assertEqual(self.redmine.attachment.get(1).export_url('pdf'), None)
 
@@ -55,7 +55,7 @@ class StandardResourcesTestCase(BaseRedmineTestCase):
         self.assertEqual(project.description, 'Bar')
 
     def test_supports_setting_of_date_datetime_attributes(self):
-        from datetime import date, datetime
+        from datetime import date, datetime, timezone
         issue = self.redmine.issue.new()
         issue.start_date = date(2014, 3, 9)
         self.assertEqual(issue.start_date, date(2014, 3, 9))
@@ -65,6 +65,12 @@ class StandardResourcesTestCase(BaseRedmineTestCase):
         self.assertEqual(issue._decoded_attrs['start_date'], '2014-03-09T20:02:02Z')
         self.assertEqual(issue._changes['start_date'], '2014-03-09T20:02:02Z')
         self.assertEqual(issue.start_date, datetime(2014, 3, 9, 20, 2, 2))
+        self.redmine.timezone = timezone.utc
+        issue.start_date = datetime(2014, 3, 9, 20, 2, 2, tzinfo=datetime.strptime('+0800', '%z').tzinfo)
+        self.assertEqual(issue._decoded_attrs['start_date'], '2014-03-09T12:02:02Z')
+        self.assertEqual(issue._changes['start_date'], '2014-03-09T12:02:02Z')
+        self.assertEqual(issue.start_date, datetime(
+            2014, 3, 9, 20, 2, 2, tzinfo=datetime.strptime('+0800', '%z').tzinfo))
 
     def test_supports_setting_of_attributes_via_dict(self):
         project = self.redmine.project.new()
@@ -144,11 +150,16 @@ class StandardResourcesTestCase(BaseRedmineTestCase):
         self.assertEqual(decoded['include'], 'a,b')
 
     def test_bulk_encode(self):
-        from datetime import date, datetime
+        from datetime import date, datetime, timezone, timedelta
         decoded = {'start_date': '2014-03-09', 'created_at': '2014-03-09T20:02:02Z'}
         encoded = self.redmine.project.resource_class.bulk_encode(decoded, self.redmine.project)
         self.assertEqual(encoded['start_date'], date(2014, 3, 9))
         self.assertEqual(encoded['created_at'], datetime(2014, 3, 9, 20, 2, 2))
+        self.redmine.timezone = datetime.strptime('+0800', '%z').tzinfo
+        encoded = self.redmine.project.resource_class.bulk_encode(decoded, self.redmine.project)
+        self.assertEqual(encoded['start_date'], date(2014, 3, 9))
+        self.assertEqual(encoded['created_at'], datetime(
+            2014, 3, 10, 4, 2, 2, tzinfo=timezone(timedelta(seconds=28800))))
 
     def test_resource_dict_is_converted_to_resource_object(self):
         self.response.json.return_value = responses['issue']['get']
@@ -193,8 +204,57 @@ class StandardResourcesTestCase(BaseRedmineTestCase):
         self.assertEqual(project.id, unpickled_project.id)
         self.assertEqual(project.name, unpickled_project.name)
 
+    def test_attach_attributes_through_registry(self):
+        class BarResource(resources.BaseResource):
+            _attach_includes = {'User': 'bars'}
+            _attach_includes_map = {'User': ('bars', 'user_bars')}
+            _attach_relations = {'Issue': 'bars', 'TimeEntry': 'bars'}
+
+        self.assertEqual(resources.registry['BarResource']['class'], BarResource)
+        self.assertIn(('bars', 'BarResource'), resources.registry['User']['attach_includes'].items())
+        self.assertIn((('bars', 'user_bars'), 'BarResource'), resources.registry['User']['attach_includes_map'].items())
+        self.assertIn(('bars', 'BarResource'), resources.registry['Issue']['attach_relations'].items())
+        self.assertIn(('bars', 'BarResource'), resources.registry['TimeEntry']['attach_relations'].items())
+        self.response.json.return_value = responses['user']['get']
+        user = self.redmine.user.get(1)
+        self.assertIn('bars', user._includes)
+        self.assertIn(('bars', 'BarResource'), user._resource_set_map.items())
+        self.assertIn(('bars', 'user_bars'), user._includes_map.items())
+        self.response.json.return_value = responses['issue']['get']
+        issue = self.redmine.issue.get(1)
+        self.assertIn('bars', issue._relations)
+        self.assertIn(('bars', 'BarResource'), issue._resource_set_map.items())
+        self.response.json.return_value = responses['time_entry']['get']
+        time_entry = self.redmine.time_entry.get(1)
+        self.assertIn('bars', time_entry._relations)
+        self.assertIn(('bars', 'BarResource'), time_entry._resource_set_map.items())
+
+    def test_attach_attributes_through_registry_before_resource_was_added(self):
+        resources.registry['BazResource'] = {'attach_relations': type('_', (), {'keys': lambda: set()})}
+
+        class BazResource(resources.BaseResource):
+            _relations = set()
+            _attach_includes = {'QuxResource': 'bazs'}
+            _attach_includes_map = {'QuxResource': ('bazs', 'qux_bazs')}
+
+        self.assertEqual(resources.registry['BazResource']['class'], BazResource)
+        self.assertEqual(resources.registry['QuxResource']['attach_includes'], {'bazs': 'BazResource'})
+        self.assertEqual(
+            resources.registry['QuxResource']['attach_includes_map'], {('bazs', 'qux_bazs'): 'BazResource'})
+
+    def test_attach_includes(self):
+        class QuxResource(resources.BaseResource):
+            _attach_includes = {'User': 'quxs'}
+            _attach_includes_map = {'User': ('quxs', 'user_quxs')}
+
+        self.response.json.return_value = {'user': {'id': 1, 'user_quxs': [{'id': 1}]}}
+        user = self.redmine.user.get(1)
+        self.assertIsInstance(user.quxs, resultsets.ResourceSet)
+        self.assertIsInstance(user.quxs[0], QuxResource)
+        self.assertEqual(user.quxs[0].id, 1)
+
     def test_project_version(self):
-        self.assertEqual(self.redmine.project.resource_class.redmine_version, '1.0')
+        self.assertEqual(self.redmine.project.resource_class.redmine_version, (1, 0, 0))
 
     def test_project_get(self):
         self.response.json.return_value = responses['project']['get']
@@ -259,9 +319,12 @@ class StandardResourcesTestCase(BaseRedmineTestCase):
         response_includes['project'].update({'enabled_modules': [{'id': 36, 'name': 'issue_tracking'}]})
         self.response.json.return_value = response_includes
         self.assertEqual(project.enabled_modules, ['issue_tracking'])
-        response_includes['project'].update({'time_entry_activities': [{'id': 1, 'name': 'developing'}]})
+        response_includes['project'].update(responses['enumeration']['filter'])
         self.response.json.return_value = response_includes
-        self.assertEqual(project.time_entry_activities, [{'id': 1, 'name': 'developing'}])
+        self.assertIsInstance(project.time_entry_activities, resultsets.ResourceSet)
+        response_includes['project'].update(responses['custom_field']['all'])
+        self.response.json.return_value = response_includes
+        self.assertIsInstance(project.issue_custom_fields, resultsets.ResourceSet)
 
     def test_project_returns_status_without_conversion(self):
         self.response.json.return_value = {'project': {'name': 'Foo', 'id': 1, 'status': 1}}
@@ -276,7 +339,7 @@ class StandardResourcesTestCase(BaseRedmineTestCase):
 
     def test_project_url(self):
         self.response.json.return_value = responses['project']['get']
-        self.assertEqual(self.redmine.project.get(1).url, '{0}/projects/foo'.format(self.url))
+        self.assertEqual(self.redmine.project.get(1).url, f'{self.url}/projects/foo')
 
     @mock.patch('redminelib.open', mock.mock_open(), create=True)
     def test_project_export(self):
@@ -290,8 +353,34 @@ class StandardResourcesTestCase(BaseRedmineTestCase):
         self.assertIsInstance(parent, resources.Project)
         self.assertEqual(parent.id, 2)
 
+    def test_project_resource_map_converts_to_resource(self):
+        self.response.json.return_value = responses['project']['get']
+        project = self.redmine.project.get(1)
+        project._decoded_attrs['default_version'] = {'id': 2, 'name': 'Release 4.0'}
+        self.assertIsInstance(project.default_version, resources.Version)
+        self.assertEqual(project.default_version.id, 2)
+        project._decoded_attrs['default_assignee'] = {'id': 4, 'name': 'John Smith'}
+        self.assertIsInstance(project.default_assignee, resources.User)
+        self.assertEqual(project.default_assignee.id, 4)
+
+    def test_project_supports_close_reopen_archive_unarchive(self):
+        self.response.json.return_value = responses['project']['get']
+        project = self.redmine.project.get(1)
+        self.response.content = ''
+        self.assertEqual(project.close(), True)
+        self.assertEqual(project.reopen(), True)
+        self.assertEqual(project.archive(), True)
+        self.assertEqual(project.unarchive(), True)
+        self.assertEqual(self.redmine.project.close(1), True)
+        self.assertEqual(self.redmine.project.reopen(1), True)
+        self.assertEqual(self.redmine.project.archive(1), True)
+        self.assertEqual(self.redmine.project.unarchive(1), True)
+        self.redmine.ver = (4, 1, 0)
+        self.assertRaises(exceptions.VersionMismatchError, lambda: project.close())
+        self.assertRaises(AttributeError, lambda: project.open())
+
     def test_issue_version(self):
-        self.assertEqual(self.redmine.issue.resource_class.redmine_version, '1.0')
+        self.assertEqual(self.redmine.issue.resource_class.redmine_version, (1, 0, 0))
 
     def test_issue_get(self):
         self.response.json.return_value = responses['issue']['get']
@@ -364,10 +453,13 @@ class StandardResourcesTestCase(BaseRedmineTestCase):
         response_includes['issue']['watchers'] = responses['user']['all']['users']
         self.response.json.return_value = response_includes
         self.assertIsInstance(issue.watchers, resultsets.ResourceSet)
+        response_includes['issue']['allowed_statuses'] = responses['issue_status']['all']['issue_statuses']
+        self.response.json.return_value = response_includes
+        self.assertIsInstance(issue.allowed_statuses, resultsets.ResourceSet)
 
     def test_issue_add_watcher_raises_exception_if_wrong_version(self):
         self.response.json.return_value = responses['issue']['get']
-        self.redmine.ver = '2.2.0'
+        self.redmine.ver = (2, 2, 0)
         issue = self.redmine.issue.get(1)
         self.assertRaises(exceptions.ResourceVersionMismatchError, lambda: issue.watcher.add(1))
 
@@ -416,6 +508,21 @@ class StandardResourcesTestCase(BaseRedmineTestCase):
         issue._decoded_attrs['journals'] = [{'id': 1}]
         self.assertEqual(issue.journals[0].url, None)
 
+    def test_issue_journals_update(self):
+        self.response.json.return_value = responses['issue']['get']
+        issue = self.redmine.issue.get(1)
+        issue._decoded_attrs['journals'] = [{'id': 1}]
+        self.response.content = ''
+        self.assertIsInstance(issue.journals[0].save(notes='new value'), resources.IssueJournal)
+        self.assertEqual(self.redmine.issue_journal.update(1, notes='new value'), True)
+
+    def test_issue_journals_resource_map_converts_to_resource(self):
+        self.response.json.return_value = responses['issue']['get']
+        issue = self.redmine.issue.get(1)
+        issue._decoded_attrs['journals'] = [{'id': 1, 'user': responses['user']['get']['user']}]
+        self.assertIsInstance(issue.journals[0].user, resources.User)
+        self.assertEqual(issue.journals[0].user.firstname, 'John')
+
     def test_issue_version_can_be_retrieved_via_version_attribute(self):
         self.response.json.return_value = {
             'issue': {'subject': 'Foo', 'id': 1, 'fixed_version': {'id': 1, 'name': 'Foo'}}}
@@ -448,7 +555,7 @@ class StandardResourcesTestCase(BaseRedmineTestCase):
 
     def test_issue_url(self):
         self.response.json.return_value = responses['issue']['get']
-        self.assertEqual(self.redmine.issue.get(1).url, '{0}/issues/1'.format(self.url))
+        self.assertEqual(self.redmine.issue.get(1).url, f'{self.url}/issues/1')
 
     @mock.patch('redminelib.open', mock.mock_open(), create=True)
     def test_issue_export(self):
@@ -464,8 +571,36 @@ class StandardResourcesTestCase(BaseRedmineTestCase):
         self.assertIsInstance(parent, resources.Issue)
         self.assertEqual(parent.id, 2)
 
+    def test_issue_resource_map_converts_to_resource(self):
+        self.response.json.return_value = responses['issue']['get']
+        issue = self.redmine.issue.get(1)
+        issue._decoded_attrs['project'] = responses['project']['get']['project']
+        self.assertIsInstance(issue.project, resources.Project)
+        self.assertEqual(issue.project.identifier, 'foo')
+        issue._decoded_attrs['tracker'] = responses['tracker']['all']['trackers'][0]
+        self.assertIsInstance(issue.tracker, resources.Tracker)
+        self.assertEqual(issue.tracker.id, 1)
+        issue._decoded_attrs['status'] = responses['issue_status']['all']['issue_statuses'][0]
+        self.assertIsInstance(issue.status, resources.IssueStatus)
+        self.assertEqual(issue.status.id, 1)
+        issue._decoded_attrs['priority'] = responses['enumeration']['filter']['time_entry_activities'][0]
+        self.assertIsInstance(issue.priority, resources.Enumeration)
+        self.assertEqual(issue.priority.id, 1)
+        issue._decoded_attrs['author'] = responses['user']['get']['user']
+        self.assertIsInstance(issue.author, resources.User)
+        self.assertEqual(issue.author.firstname, 'John')
+        issue._decoded_attrs['assigned_to'] = responses['user']['get']['user']
+        self.assertIsInstance(issue.assigned_to, resources.User)
+        self.assertEqual(issue.assigned_to.lastname, 'Smith')
+        issue._decoded_attrs['category'] = responses['issue_category']['get']['issue_category']
+        self.assertIsInstance(issue.category, resources.IssueCategory)
+        self.assertEqual(issue.category.id, 1)
+        issue._decoded_attrs['fixed_version'] = responses['version']['get']['version']
+        self.assertIsInstance(issue.fixed_version, resources.Version)
+        self.assertEqual(issue.fixed_version.id, 1)
+
     def test_time_entry_version(self):
-        self.assertEqual(self.redmine.time_entry.resource_class.redmine_version, '1.1')
+        self.assertEqual(self.redmine.time_entry.resource_class.redmine_version, (1, 1, 0))
 
     def test_time_entry_get(self):
         self.response.json.return_value = responses['time_entry']['get']
@@ -534,7 +669,7 @@ class StandardResourcesTestCase(BaseRedmineTestCase):
 
     def test_time_entry_url(self):
         self.response.json.return_value = responses['time_entry']['get']
-        self.assertEqual(self.redmine.time_entry.get(1).url, '{0}/time_entries/1'.format(self.url))
+        self.assertEqual(self.redmine.time_entry.get(1).url, f'{self.url}/time_entries/1')
 
     @mock.patch('redminelib.open', mock.mock_open(), create=True)
     def test_time_entry_export(self):
@@ -542,8 +677,24 @@ class StandardResourcesTestCase(BaseRedmineTestCase):
         self.response.iter_content = lambda chunk_size: (str(num) for num in range(0, 5))
         self.assertEqual(self.redmine.time_entry.all().export('txt', '/foo/bar'), '/foo/bar/time_entries.txt')
 
+    def test_time_entry_resource_map_converts_to_resource(self):
+        self.response.json.return_value = responses['time_entry']['get']
+        time_entry = self.redmine.time_entry.get(1)
+        time_entry._decoded_attrs['project'] = responses['project']['get']['project']
+        self.assertIsInstance(time_entry.project, resources.Project)
+        self.assertEqual(time_entry.project.identifier, 'foo')
+        time_entry._decoded_attrs['issue'] = responses['issue']['get']['issue']
+        self.assertIsInstance(time_entry.issue, resources.Issue)
+        self.assertEqual(time_entry.issue.id, 1)
+        time_entry._decoded_attrs['user'] = responses['user']['get']['user']
+        self.assertIsInstance(time_entry.user, resources.User)
+        self.assertEqual(time_entry.user.firstname, 'John')
+        time_entry._decoded_attrs['activity'] = responses['enumeration']['filter']['time_entry_activities'][0]
+        self.assertIsInstance(time_entry.activity, resources.Enumeration)
+        self.assertEqual(time_entry.activity.id, 1)
+
     def test_enumeration_version(self):
-        self.assertEqual(self.redmine.enumeration.resource_class.redmine_version, '2.2')
+        self.assertEqual(self.redmine.enumeration.resource_class.redmine_version, (2, 2, 0))
 
     def test_enumeration_get(self):
         self.response.json.return_value = responses['enumeration']['filter']
@@ -563,11 +714,11 @@ class StandardResourcesTestCase(BaseRedmineTestCase):
         self.response.json.return_value = responses['enumeration']['filter']
         self.assertEqual(
             self.redmine.enumeration.filter(resource='time_entry_activities')[0].url,
-            '{0}/enumerations/1/edit'.format(self.url)
+            f'{self.url}/enumerations/1/edit'
         )
 
     def test_attachment_version(self):
-        self.assertEqual(self.redmine.attachment.resource_class.redmine_version, '1.3')
+        self.assertEqual(self.redmine.attachment.resource_class.redmine_version, (1, 3, 0))
 
     def test_attachment_get(self):
         self.response.json.return_value = responses['attachment']['get']
@@ -610,7 +761,7 @@ class StandardResourcesTestCase(BaseRedmineTestCase):
 
     def test_attachment_url(self):
         self.response.json.return_value = responses['attachment']['get']
-        self.assertEqual(self.redmine.attachment.get(1).url, '{0}/attachments/1'.format(self.url))
+        self.assertEqual(self.redmine.attachment.get(1).url, f'{self.url}/attachments/1')
 
     @mock.patch('redminelib.open', mock.mock_open(), create=True)
     def test_attachment_download(self):
@@ -620,8 +771,15 @@ class StandardResourcesTestCase(BaseRedmineTestCase):
         self.response.iter_content = lambda chunk_size: (str(num) for num in range(0, 5))
         self.assertEqual(self.redmine.attachment.get(1).download('/some/path'), '/some/path/bar.txt')
 
+    def test_attachment_resource_map_converts_to_resource(self):
+        self.response.json.return_value = responses['attachment']['get']
+        attachment = self.redmine.attachment.get(1)
+        attachment._decoded_attrs['author'] = responses['user']['get']['user']
+        self.assertIsInstance(attachment.author, resources.User)
+        self.assertEqual(attachment.author.firstname, 'John')
+
     def test_file_version(self):
-        self.assertEqual(self.redmine.file.resource_class.redmine_version, '3.4')
+        self.assertEqual(self.redmine.file.resource_class.redmine_version, (3, 4, 0))
 
     def test_file_get(self):
         self.response.json.return_value = responses['attachment']['get']
@@ -684,7 +842,7 @@ class StandardResourcesTestCase(BaseRedmineTestCase):
 
     def test_file_url(self):
         self.response.json.return_value = responses['attachment']['get']
-        self.assertEqual(self.redmine.file.get(1).url, '{0}/attachments/1'.format(self.url))
+        self.assertEqual(self.redmine.file.get(1).url, f'{self.url}/attachments/1')
 
     @mock.patch('redminelib.open', mock.mock_open(), create=True)
     def test_file_download(self):
@@ -694,8 +852,18 @@ class StandardResourcesTestCase(BaseRedmineTestCase):
         self.response.iter_content = lambda chunk_size: (str(num) for num in range(0, 5))
         self.assertEqual(self.redmine.file.get(1).download('/some/path'), '/some/path/bar.txt')
 
+    def test_file_resource_map_converts_to_resource(self):
+        self.response.json.return_value = responses['attachment']['get']
+        f = self.redmine.file.get(1)
+        f._decoded_attrs['author'] = responses['user']['get']['user']
+        self.assertIsInstance(f.author, resources.User)
+        self.assertEqual(f.author.firstname, 'John')
+        f._decoded_attrs['version'] = responses['version']['get']['version']
+        self.assertIsInstance(f.version, resources.Version)
+        self.assertEqual(f.version.id, 1)
+
     def test_wiki_page_version(self):
-        self.assertEqual(self.redmine.wiki_page.resource_class.redmine_version, '2.2')
+        self.assertEqual(self.redmine.wiki_page.resource_class.redmine_version, (2, 2, 0))
 
     def test_wiki_page_get(self):
         self.response.json.return_value = responses['wiki_page']['get']
@@ -706,10 +874,7 @@ class StandardResourcesTestCase(BaseRedmineTestCase):
         """Test getting a wiki page with special char in title."""
         self.response.json.return_value = responses['wiki_page']['get_special']
         wiki_page = self.redmine.wiki_page.get('Foo%Bar', project_id=1)
-        self.assertEqual(
-            self.patch_requests.call_args[0][1],
-            '{0}/projects/1/wiki/Foo%25Bar.json'.format(self.url)
-        )
+        self.assertEqual(self.patch_requests.call_args[0][1], f'{self.url}/projects/1/wiki/Foo%25Bar.json')
         self.assertEqual(wiki_page.title, 'Foo%Bar')
         self.assertEqual(wiki_page.url, 'http://foo.bar/projects/1/wiki/Foo%25Bar')
 
@@ -730,10 +895,7 @@ class StandardResourcesTestCase(BaseRedmineTestCase):
         self.response.status_code = 201
         self.response.json.return_value = responses['wiki_page']['get_special']
         wiki_page = self.redmine.wiki_page.create(project_id='foo', title='Foo%Bar')
-        self.assertEqual(
-            self.patch_requests.call_args[0][1],
-            '{0}/projects/foo/wiki/Foo%25Bar.json'.format(self.url)
-        )
+        self.assertEqual(self.patch_requests.call_args[0][1], f'{self.url}/projects/foo/wiki/Foo%25Bar.json')
         self.assertEqual(wiki_page.title, 'Foo%Bar')
 
     def test_wiki_page_delete(self):
@@ -797,10 +959,7 @@ class StandardResourcesTestCase(BaseRedmineTestCase):
 
     def test_wiki_page_url(self):
         self.response.json.return_value = responses['wiki_page']['get']
-        self.assertEqual(
-            self.redmine.wiki_page.get('Foo', project_id='Foo').url,
-            '{0}/projects/Foo/wiki/Foo'.format(self.url)
-        )
+        self.assertEqual(self.redmine.wiki_page.get('Foo', project_id='Foo').url, f'{self.url}/projects/Foo/wiki/Foo')
 
     @mock.patch('redminelib.open', mock.mock_open(), create=True)
     def test_wiki_page_export(self):
@@ -814,8 +973,15 @@ class StandardResourcesTestCase(BaseRedmineTestCase):
         self.assertIsInstance(parent, resources.WikiPage)
         self.assertEqual(parent.title, 'Bar')
 
+    def test_wiki_page_resource_map_converts_to_resource(self):
+        self.response.json.return_value = responses['wiki_page']['get']
+        wiki_page = self.redmine.wiki_page.get('Foo', project_id=1)
+        wiki_page._decoded_attrs['author'] = responses['user']['get']['user']
+        self.assertIsInstance(wiki_page.author, resources.User)
+        self.assertEqual(wiki_page.author.firstname, 'John')
+
     def test_project_membership_version(self):
-        self.assertEqual(self.redmine.project_membership.resource_class.redmine_version, '1.4')
+        self.assertEqual(self.redmine.project_membership.resource_class.redmine_version, (1, 4, 0))
 
     def test_project_membership_get(self):
         self.response.json.return_value = responses['project_membership']['get']
@@ -865,10 +1031,23 @@ class StandardResourcesTestCase(BaseRedmineTestCase):
 
     def test_project_membership_url(self):
         self.response.json.return_value = responses['project_membership']['get']
-        self.assertEqual(self.redmine.project_membership.get(1).url, '{0}/memberships/1'.format(self.url))
+        self.assertEqual(self.redmine.project_membership.get(1).url, f'{self.url}/memberships/1')
+
+    def test_project_membership_resource_map_converts_to_resource(self):
+        self.response.json.return_value = responses['project_membership']['get']
+        membership = self.redmine.project_membership.get(1)
+        membership._decoded_attrs['project'] = responses['project']['get']['project']
+        self.assertIsInstance(membership.project, resources.Project)
+        self.assertEqual(membership.project.identifier, 'foo')
+        membership._decoded_attrs['user'] = responses['user']['get']['user']
+        self.assertIsInstance(membership.user, resources.User)
+        self.assertEqual(membership.user.firstname, 'John')
+        membership._decoded_attrs['group'] = responses['group']['get']['group']
+        self.assertIsInstance(membership.group, resources.Group)
+        self.assertEqual(membership.group.id, 1)
 
     def test_issue_category_version(self):
-        self.assertEqual(self.redmine.issue_category.resource_class.redmine_version, '1.3')
+        self.assertEqual(self.redmine.issue_category.resource_class.redmine_version, (1, 3, 0))
 
     def test_issue_category_get(self):
         self.response.json.return_value = responses['issue_category']['get']
@@ -911,10 +1090,20 @@ class StandardResourcesTestCase(BaseRedmineTestCase):
 
     def test_issue_category_url(self):
         self.response.json.return_value = responses['issue_category']['get']
-        self.assertEqual(self.redmine.issue_category.get(1).url, '{0}/issue_categories/1'.format(self.url))
+        self.assertEqual(self.redmine.issue_category.get(1).url, f'{self.url}/issue_categories/1')
+
+    def test_issue_category_resource_map_converts_to_resource(self):
+        self.response.json.return_value = responses['issue_category']['get']
+        category = self.redmine.issue_category.get(1)
+        category._decoded_attrs['project'] = responses['project']['get']['project']
+        self.assertIsInstance(category.project, resources.Project)
+        self.assertEqual(category.project.identifier, 'foo')
+        category._decoded_attrs['assigned_to'] = responses['user']['get']['user']
+        self.assertIsInstance(category.assigned_to, resources.User)
+        self.assertEqual(category.assigned_to.firstname, 'John')
 
     def test_issue_relation_version(self):
-        self.assertEqual(self.redmine.issue_relation.resource_class.redmine_version, '1.3')
+        self.assertEqual(self.redmine.issue_relation.resource_class.redmine_version, (1, 3, 0))
 
     def test_issue_relation_get(self):
         self.response.json.return_value = responses['issue_relation']['get']
@@ -956,10 +1145,10 @@ class StandardResourcesTestCase(BaseRedmineTestCase):
 
     def test_issue_relation_url(self):
         self.response.json.return_value = responses['issue_relation']['get']
-        self.assertEqual(self.redmine.issue_relation.get(1).url, '{0}/relations/1'.format(self.url))
+        self.assertEqual(self.redmine.issue_relation.get(1).url, f'{self.url}/relations/1')
 
     def test_version_version(self):
-        self.assertEqual(self.redmine.version.resource_class.redmine_version, '1.3')
+        self.assertEqual(self.redmine.version.resource_class.redmine_version, (1, 3, 0))
 
     def test_version_get(self):
         self.response.json.return_value = responses['version']['get']
@@ -1007,10 +1196,17 @@ class StandardResourcesTestCase(BaseRedmineTestCase):
 
     def test_version_url(self):
         self.response.json.return_value = responses['version']['get']
-        self.assertEqual(self.redmine.version.get(1).url, '{0}/versions/1'.format(self.url))
+        self.assertEqual(self.redmine.version.get(1).url, f'{self.url}/versions/1')
+
+    def test_version_resource_map_converts_to_resource(self):
+        self.response.json.return_value = responses['version']['get']
+        version = self.redmine.version.get(1)
+        version._decoded_attrs['project'] = responses['project']['get']['project']
+        self.assertIsInstance(version.project, resources.Project)
+        self.assertEqual(version.project.identifier, 'foo')
 
     def test_user_version(self):
-        self.assertEqual(self.redmine.user.resource_class.redmine_version, '1.1')
+        self.assertEqual(self.redmine.user.resource_class.redmine_version, (1, 1, 0))
 
     def test_user_get(self):
         self.response.json.return_value = responses['user']['get']
@@ -1116,10 +1312,10 @@ class StandardResourcesTestCase(BaseRedmineTestCase):
 
     def test_user_url(self):
         self.response.json.return_value = responses['user']['get']
-        self.assertEqual(self.redmine.user.get(1).url, '{0}/users/1'.format(self.url))
+        self.assertEqual(self.redmine.user.get(1).url, f'{self.url}/users/1')
 
     def test_group_version(self):
-        self.assertEqual(self.redmine.group.resource_class.redmine_version, '2.1')
+        self.assertEqual(self.redmine.group.resource_class.redmine_version, (2, 1, 0))
 
     def test_group_get(self):
         self.response.json.return_value = responses['group']['get']
@@ -1185,10 +1381,10 @@ class StandardResourcesTestCase(BaseRedmineTestCase):
 
     def test_group_url(self):
         self.response.json.return_value = responses['group']['get']
-        self.assertEqual(self.redmine.group.get(1).url, '{0}/groups/1'.format(self.url))
+        self.assertEqual(self.redmine.group.get(1).url, f'{self.url}/groups/1')
 
     def test_role_version(self):
-        self.assertEqual(self.redmine.role.resource_class.redmine_version, '1.4')
+        self.assertEqual(self.redmine.role.resource_class.redmine_version, (1, 4, 0))
 
     def test_role_get(self):
         self.response.json.return_value = responses['role']['get']
@@ -1206,10 +1402,10 @@ class StandardResourcesTestCase(BaseRedmineTestCase):
 
     def test_role_url(self):
         self.response.json.return_value = responses['role']['get']
-        self.assertEqual(self.redmine.role.get(1).url, '{0}/roles/1'.format(self.url))
+        self.assertEqual(self.redmine.role.get(1).url, f'{self.url}/roles/1')
 
     def test_news_version(self):
-        self.assertEqual(self.redmine.news.resource_class.redmine_version, '1.1')
+        self.assertEqual(self.redmine.news.resource_class.redmine_version, (1, 1, 0))
 
     def test_news_get(self):
         self.response.json.return_value = responses['news']['get']
@@ -1262,7 +1458,7 @@ class StandardResourcesTestCase(BaseRedmineTestCase):
 
     def test_news_url(self):
         self.response.json.return_value = responses['news']['filter']
-        self.assertEqual(self.redmine.news.filter(project_id=1)[0].url, '{0}/news/2'.format(self.url))
+        self.assertEqual(self.redmine.news.filter(project_id=1)[0].url, f'{self.url}/news/2')
 
     @mock.patch('redminelib.open', mock.mock_open(), create=True)
     def test_news_export(self):
@@ -1278,8 +1474,29 @@ class StandardResourcesTestCase(BaseRedmineTestCase):
         self.response.json.return_value = responses['news']['filter']
         self.assertEqual(repr(self.redmine.news.filter(project_id=1)[0]), '<redminelib.resources.News #2 "Foo">')
 
+    def test_news_resource_map_converts_to_resource(self):
+        self.response.json.return_value = responses['news']['get']
+        news = self.redmine.news.get(1)
+        news._decoded_attrs['project'] = responses['project']['get']['project']
+        self.assertIsInstance(news.project, resources.Project)
+        self.assertEqual(news.project.identifier, 'foo')
+        news._decoded_attrs['author'] = responses['user']['get']['user']
+        self.assertIsInstance(news.author, resources.User)
+        self.assertEqual(news.author.firstname, 'John')
+
+    def test_news_includes(self):
+        response_includes = responses['news']['get']
+        self.response.json.return_value = response_includes
+        news = self.redmine.news.get(1)
+        response_includes['news'].update({'attachments': [responses['attachment']['get']['attachment']]})
+        self.response.json.return_value = response_includes
+        self.assertIsInstance(news.attachments, resultsets.ResourceSet)
+        response_includes['news'].update({'comments': [{'id': 1, 'content': 'foobar'}]})
+        self.response.json.return_value = response_includes
+        self.assertIsInstance(news.comments, list)
+
     def test_issue_status_version(self):
-        self.assertEqual(self.redmine.issue_status.resource_class.redmine_version, '1.3')
+        self.assertEqual(self.redmine.issue_status.resource_class.redmine_version, (1, 3, 0))
 
     def test_issue_status_get(self):
         self.response.json.return_value = responses['issue_status']['all']
@@ -1297,10 +1514,10 @@ class StandardResourcesTestCase(BaseRedmineTestCase):
 
     def test_issue_status_url(self):
         self.response.json.return_value = responses['issue_status']['all']
-        self.assertEqual(self.redmine.issue_status.all()[0].url, '{0}/issue_statuses/1/edit'.format(self.url))
+        self.assertEqual(self.redmine.issue_status.all()[0].url, f'{self.url}/issue_statuses/1/edit')
 
     def test_tracker_version(self):
-        self.assertEqual(self.redmine.tracker.resource_class.redmine_version, '1.3')
+        self.assertEqual(self.redmine.tracker.resource_class.redmine_version, (1, 3, 0))
 
     def test_tracker_get(self):
         self.response.json.return_value = responses['tracker']['all']
@@ -1318,10 +1535,10 @@ class StandardResourcesTestCase(BaseRedmineTestCase):
 
     def test_tracker_url(self):
         self.response.json.return_value = responses['tracker']['all']
-        self.assertEqual(self.redmine.tracker.all()[0].url, '{0}/trackers/1/edit'.format(self.url))
+        self.assertEqual(self.redmine.tracker.all()[0].url, f'{self.url}/trackers/1/edit')
 
     def test_query_version(self):
-        self.assertEqual(self.redmine.query.resource_class.redmine_version, '1.3')
+        self.assertEqual(self.redmine.query.resource_class.redmine_version, (1, 3, 0))
 
     def test_query_get(self):
         self.response.json.return_value = responses['query']['all']
@@ -1339,10 +1556,10 @@ class StandardResourcesTestCase(BaseRedmineTestCase):
 
     def test_query_url(self):
         self.response.json.return_value = responses['query']['all']
-        self.assertEqual(self.redmine.query.all()[0].url, '{0}/projects/0/issues?query_id=1'.format(self.url))
+        self.assertEqual(self.redmine.query.all()[0].url, f'{self.url}/projects/0/issues?query_id=1')
 
     def test_custom_field_version(self):
-        self.assertEqual(self.redmine.custom_field.resource_class.redmine_version, '2.4')
+        self.assertEqual(self.redmine.custom_field.resource_class.redmine_version, (2, 4, 0))
 
     def test_custom_field_get(self):
         self.response.json.return_value = responses['custom_field']['all']
@@ -1374,4 +1591,4 @@ class StandardResourcesTestCase(BaseRedmineTestCase):
 
     def test_custom_field_url(self):
         self.response.json.return_value = responses['custom_field']['all']
-        self.assertEqual(self.redmine.custom_field.all()[0].url, '{0}/custom_fields/1/edit'.format(self.url))
+        self.assertEqual(self.redmine.custom_field.all()[0].url, f'{self.url}/custom_fields/1/edit')
